package java.util.zip;

import java.io.EOFException;
import java.io.InputStream;
import java.io.IOException;
import java.io.FilterInputStream;

public class InflaterInputStream extends FilterInputStream
{
    protected Inflater inf;
    protected byte[] buf;
    protected int len;
    private boolean closed;
    private boolean reachEOF;
    boolean usesDefaultInflater;
    private byte[] singleByteBuf;
    private byte[] b;
    
    private void ensureOpen() throws IOException {
        if (this.closed) {
            throw new IOException("Stream closed");
        }
        return;
    }
    
    public InflaterInputStream(InputStream in, Inflater inf, int n) {
        super(in);
        this.closed = false;
        this.reachEOF = false;
        this.usesDefaultInflater = false;
        this.singleByteBuf = new byte[1];
        this.b = new byte[512];
        if (in == null || inf == null) {
            throw new NullPointerException();
        }
        if (n <= 0) {
            throw new IllegalArgumentException("buffer size <= 0");
        }
        this.inf = inf;
        this.buf = new byte[n];
        return;
    }
    
    public InflaterInputStream(InputStream inputStream, Inflater inflater) {
        this(inputStream, inflater, 512);
    }
    
    public InflaterInputStream(InputStream inputStream) {
        this(inputStream, new Inflater());
        this.usesDefaultInflater = true;
    }
    
    @Override
    public int read() throws IOException {
        this.ensureOpen();
        return (this.read(this.singleByteBuf, 0, 1) == -1) ? -1 : Byte.toUnsignedInt(this.singleByteBuf[0]);
    }
    
    @Override
    public int read(byte[] array, int n, int n2) throws IOException {
        this.ensureOpen();
        if (array == null) {
            throw new NullPointerException();
        }
        if (n < 0 || n2 < 0 || n2 > array.length - n) {
            throw new IndexOutOfBoundsException();
        }
        if (n2 == 0) {
            return 0;
        }
        try {
            int inflate;
            while ((inflate = this.inf.inflate(array, n, n2)) == 0) {
                if (this.inf.finished() || this.inf.needsDictionary()) {
                    this.reachEOF = true;
                    return -1;
                }
                if (!this.inf.needsInput()) {
                    continue;
                }
                this.fill();
            }
            return inflate;
        }
        catch (DataFormatException ex) {
            String message = ex.getMessage();
            throw new ZipException((message != null) ? message : "Invalid ZLIB data format");
        }
    }
    
    @Override
    public int available() throws IOException {
        this.ensureOpen();
        if (this.reachEOF) {
            return 0;
        }
        return 1;
    }
    
    @Override
    public long skip(long a) throws IOException {
        if (a < 0L) {
            throw new IllegalArgumentException("negative skip length");
        }
        this.ensureOpen();
        int n = (int)Math.min(a, 2147483647L);
        int i = 0;
        while (i < n) {
            int length = n - i;
            if (length > this.b.length) {
                length = this.b.length;
            }
            int read = this.read(this.b, 0, length);
            if (read == -1) {
                this.reachEOF = true;
                break;
            }
            i += read;
        }
        return (long)i;
    }
    
    @Override
    public void close() throws IOException {
        if (!this.closed) {
            if (this.usesDefaultInflater) {
                this.inf.end();
            }
            this.in.close();
            this.closed = true;
        }
    }
    
    protected void fill() throws IOException {
        this.ensureOpen();
        this.len = this.in.read(this.buf, 0, this.buf.length);
        if (this.len == -1) {
            throw new EOFException("Unexpected end of ZLIB input stream");
        }
        this.inf.setInput(this.buf, 0, this.len);
        return;
    }
    
    @Override
    public boolean markSupported() {
        return false;
    }
    
    @Override
    public synchronized void mark(int n) {
    }
    
    @Override
    public synchronized void reset() throws IOException {
        throw new IOException("mark/reset not supported");
    }
}
